
var HE = {};

HE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

HE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { HE.backuprange(); });

// Initializations
HE.callback = function() {
    window.location.href = "he-callback://" + encodeURIComponent(HE.getHtml());
}

HE.setHtml = function(contents) {
    HE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

HE.getHtml = function() {
    return HE.editor.innerHTML;
}

HE.getText = function() {
    return HE.editor.innerText;
}

HE.setBaseTextColor = function(color) {
    HE.editor.style.color  = color;
}

HE.setBaseFontSize = function(size) {
    HE.editor.style.fontSize = size;
}

HE.setPadding = function(left, top, right, bottom) {
  HE.editor.style.paddingLeft = left;
  HE.editor.style.paddingTop = top;
  HE.editor.style.paddingRight = right;
  HE.editor.style.paddingBottom = bottom;
}

HE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

HE.setBackgroundImage = function(image) {
    HE.editor.style.backgroundImage = image;
}

HE.setWidth = function(size) {
    HE.editor.style.minWidth = size;
}

HE.setHeight = function(size) {
    HE.editor.style.height = size;
}

HE.setTextAlign = function(align) {
    HE.editor.style.textAlign = align;
}

HE.setVerticalAlign = function(align) {
    HE.editor.style.verticalAlign = align;
}

HE.setPlaceholder = function(placeholder) {
    HE.editor.setAttribute("placeholder", placeholder);
}

HE.setInputEnabled = function(inputEnabled) {
    HE.editor.contentEditable = String(inputEnabled);
}

HE.undo = function() {
    document.execCommand('undo', false, null);
    HE.enabledEditingItems();
}

HE.redo = function() {
    document.execCommand('redo', false, null);
    HE.enabledEditingItems();
}

HE.setBold = function() {
    document.execCommand('bold', false, null);
    HE.enabledEditingItems();
}

HE.setItalic = function() {
    document.execCommand('italic', false, null);
    HE.enabledEditingItems();
}

HE.setSubscript = function() {
    /* FIXME: subscript is not toggled  */
    if (document.queryCommandState('subscript')) {
        document.execCommand('unscript', false, null);
    }
    else {
    document.execCommand('subscript', false, null);
    }
    HE.enabledEditingItems();
}

HE.setSuperscript = function() {
    /* FIXME: superscript is not toggled  */
    if (document.queryCommandState('superscript')) {
       document.execCommand('unscript', false, null);
    }
    else {
       document.execCommand('superscript', false, null);
    }
    HE.enabledEditingItems();
}

HE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
    HE.enabledEditingItems();
}

HE.setUnderline = function() {
    document.execCommand('underline', false, null);
    HE.enabledEditingItems();
}

HE.setBullets = function() {
     /* Toggle off numbered list not working with bullets   */
     if ( document.queryCommandState('insertOrderedList') ) {
         document.execCommand('insertOrderedList', false, null);
     }

    document.execCommand('insertUnorderedList', false, null);
    HE.enabledEditingItems();
}

HE.setNumbers = function() {
    /* Toggle off bullets not working with bullets   */
    if ( document.queryCommandState('insertUnorderedList') ) {
        document.execCommand('insertUnorderedList', false, null);
    }

    document.execCommand('insertOrderedList', false, null);
    HE.enabledEditingItems();
}

HE.setTextColor = function(color) {
    HE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
    HE.enabledEditingItems();
}

HE.setTextBackgroundColor = function(color) {
    /* This is workaround to toggle highlight          */
    var bgColor = document.queryCommandValue('backColor');
    if ( bgColor != null
            && bgColor.toUpperCase().replace(/\s/g, '') != 'RGBA(0,0,0,0)'
            && bgColor.toUpperCase().replace(/\s/g, '') != 'RGBA(255,255,255,0)'
            && bgColor.toUpperCase().replace(/\s/g, '') != 'RGB(0,0,0)'
            && bgColor.toUpperCase().replace(/\s/g, '') != 'RGB(255,255,255)') {
        color = '#FFFFFF';
    }
    HE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);

    HE.enabledEditingItems();
}

HE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
    HE.enabledEditingItems();
}


HE.setHeading = function(heading) {
     /* Toggle off styles not working with heading */
     if ( document.queryCommandState('bold') ) {
         document.execCommand('bold', false, null);
     }
     if ( document.queryCommandState('insertOrderedList') ) {
         document.execCommand('insertOrderedList', false, null);
     }
     if ( document.queryCommandState('insertUnorderedList') ) {
         document.execCommand('insertUnorderedList', false, null);
     }

    document.execCommand('formatBlock', false, '<h'+heading+'>');
    HE.enabledEditingItems();
}

HE.setIndent = function() {
    document.execCommand('indent', false, null);
    HE.enabledEditingItems();
}

HE.setOutdent = function() {
    document.execCommand('outdent', false, null);
    HE.enabledEditingItems();
}

HE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
    HE.enabledEditingItems();
}

HE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
    HE.enabledEditingItems();
}

HE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
    HE.enabledEditingItems();
}

HE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
    HE.enabledEditingItems();
}

HE.insertImage = function(url, alt) {
    var html = '<img src="' + url + '" alt="' + alt + '" />';
    HE.insertHTML(html);
}

HE.insertImageW = function(url, alt, width) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '"/>';
    HE.insertHTML(html);
}

HE.insertImageWH = function(url, alt, width, height) {
    var html = '<img src="' + url + '" alt="' + alt + '" width="' + width + '" height="' + height +'"/>';
    HE.insertHTML(html);
}

HE.insertVideo = function(url, alt) {
    var html = '<video src="' + url + '" controls></video><br>';
    HE.insertHTML(html);
}

HE.insertVideoW = function(url, width) {
    var html = '<video src="' + url + '" width="' + width + '" controls></video><br>';
    HE.insertHTML(html);
}

HE.insertVideoWH = function(url, width, height) {
    var html = '<video src="' + url + '" width="' + width + '" height="' + height + '" controls></video><br>';
    HE.insertHTML(html);
}

HE.insertAudio = function(url, alt) {
    var html = '<audio src="' + url + '" controls></audio><br>';
    HE.insertHTML(html);
}

HE.insertYoutubeVideo = function(url) {
    var html = '<iframe width="100%" height="100%" src="' + url +
    '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; ' +
    'picture-in-picture" allowfullscreen></iframe><br>'
    HE.insertHTML(html);
}

HE.insertYoutubeVideoW = function(url, width) {
    var html = '<iframe width="' + width + '" src="' + url + '" frameborder="0" allow="accelerometer; ' +
    'autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><br>'
    HE.insertHTML(html);
}

HE.insertYoutubeVideoWH = function(url, width, height) {
    var html = '<iframe width="' + width + '" height="' + height + '" src="' + url +
    '" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; '
    + 'picture-in-picture" allowfullscreen></iframe><br>'
    HE.insertHTML(html);
}

HE.insertHTML = function(html) {
    HE.restorerange();
    document.execCommand('insertHTML', false, html);
}

HE.insertLink = function(url, title) {
    HE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML",false,"<a href='"+url+"'>"+title+"</a>");
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
    HE.callback();
}

HE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

HE.prepareInsert = function() {
    HE.backuprange();
}

HE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      HE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

HE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(HE.currentSelection.startContainer, HE.currentSelection.startOffset);
    range.setEnd(HE.currentSelection.endContainer, HE.currentSelection.endOffset);
    selection.addRange(range);
}

HE.enabledEditingItems = function(e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }
    var textBgColor = document.queryCommandValue('backColor');
    if ( textBgColor != null
            && textBgColor.toUpperCase().replace(/\s/g, '') != 'RGBA(0,0,0,0)'
            && textBgColor.toUpperCase().replace(/\s/g, '') != 'RGBA(255,255,255,0)'
            && textBgColor.toUpperCase().replace(/\s/g, '') != 'RGB(0,0,0)'
            && textBgColor.toUpperCase().replace(/\s/g, '') != 'RGB(255,255,255)') {
        // highlight is toggle off
    }
    else {
        items.push("highlight");
    }

    var textColor = document.queryCommandValue('foreColor');
    items.push('textColor:' + textColor);

    window.location.href = "he-state://" + encodeURI(items.join('|'));
}

HE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(HE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    HE.editor.focus();
}

HE.blurFocus = function() {
    HE.editor.blur();
}

HE.removeFormat = function() {
    document.execCommand('removeFormat', false, null);
}

// Event Listeners
HE.editor.addEventListener("input", HE.callback);
HE.editor.addEventListener("keyup", function(e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39, KEY_ENTER = 13;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT || e.which == KEY_ENTER) {
        HE.enabledEditingItems(e);
    }
});
HE.editor.addEventListener("click", HE.enabledEditingItems);
